@@ -290,7 +290,7 @@ gst_ffmpegvidenc_set_format (GstVideoEncoder * encoder,
   GstCaps *allowed_caps;
   GstCaps *icaps;
   GstVideoCodecState *output_format;
-  enum PixelFormat pix_fmt;
+  enum AVPixelFormat pix_fmt;
   GstFFMpegVidEnc *ffmpegenc = (GstFFMpegVidEnc *) encoder;
   GstFFMpegVidEncClass *oclass =
       (GstFFMpegVidEncClass *) G_OBJECT_GET_CLASS (ffmpegenc);
@@ -430,6 +430,29 @@ gst_ffmpegvidenc_set_format (GstVideoEncoder * encoder,
   gst_ffmpeg_caps_with_codecid (oclass->in_plugin->id,
       oclass->in_plugin->type, allowed_caps, ffmpegenc->context);
 
+  /* open codec */
+  if (gst_ffmpeg_avcodec_open (ffmpegenc->context, oclass->in_plugin) < 0) {
+    gst_caps_unref (allowed_caps);
+    goto open_codec_fail;
+  }
+
+  /* is the colourspace correct? */
+  if (pix_fmt != ffmpegenc->context->pix_fmt) {
+    gst_caps_unref (allowed_caps);
+    goto pix_fmt_err;
+  }
+
+  /* we may have failed mapping caps to a pixfmt,
+   * and quite some codecs do not make up their own mind about that
+   * in any case, _NONE can never work out later on */
+  if (pix_fmt == AV_PIX_FMT_NONE) {
+    gst_caps_unref (allowed_caps);
+    goto bad_input_fmt;
+  }
+
+  /* second pass stats buffer no longer needed */
+  g_free (ffmpegenc->context->stats_in);
+
   /* try to set this caps on the other side */
   other_caps = gst_ffmpeg_codecid_to_caps (oclass->in_plugin->id,
       ffmpegenc->context, TRUE);
@@ -450,24 +473,6 @@ gst_ffmpegvidenc_set_format (GstVideoEncoder * encoder,
 
   GST_DEBUG_OBJECT (ffmpegenc, "codec flags 0x%08x", ffmpegenc->context->flags);
 
-  /* open codec */
-  if (gst_ffmpeg_avcodec_open (ffmpegenc->context, oclass->in_plugin) < 0)
-    goto open_codec_fail;
-
-  /* is the colourspace correct? */
-  if (pix_fmt != ffmpegenc->context->pix_fmt)
-    goto pix_fmt_err;
-
-  /* we may have failed mapping caps to a pixfmt,
-   * and quite some codecs do not make up their own mind about that
-   * in any case, _NONE can never work out later on */
-  if (pix_fmt == AV_PIX_FMT_NONE)
-    goto bad_input_fmt;
-
-  /* second pass stats buffer no longer needed */
-  if (ffmpegenc->context->stats_in)
-    g_free (ffmpegenc->context->stats_in);
-
   /* Store input state and set output state */
   if (ffmpegenc->input_state)
     gst_video_codec_state_unref (ffmpegenc->input_state);
@@ -482,7 +487,7 @@ gst_ffmpegvidenc_set_format (GstVideoEncoder * encoder,
     const gchar *codec;
 
     gst_tag_list_add (tags, GST_TAG_MERGE_REPLACE, GST_TAG_NOMINAL_BITRATE,
-        ffmpegenc->context->bit_rate, NULL);
+        (guint) ffmpegenc->context->bit_rate, NULL);
 
     if ((codec =
             gst_ffmpeg_get_codecid_longname (ffmpegenc->context->codec_id)))
@@ -556,8 +561,7 @@ close_codec:
   }
 cleanup_stats_in:
   {
-    if (ffmpegenc->context->stats_in)
-      g_free (ffmpegenc->context->stats_in);
+    g_free (ffmpegenc->context->stats_in);
     return FALSE;
   }
 }
@@ -626,6 +630,7 @@ gst_ffmpegvidenc_handle_frame (GstVideoEncoder * encoder,
     GST_ERROR_OBJECT (encoder, "Failed to map input buffer");
     gst_buffer_unref (buffer_info->buffer);
     g_slice_free (BufferInfo, buffer_info);
+    gst_video_codec_frame_unref (frame);
     return GST_FLOW_ERROR;
   }
 
@@ -644,6 +649,10 @@ gst_ffmpegvidenc_handle_frame (GstVideoEncoder * encoder,
     }
   }
 
+  ffmpegenc->picture->format = ffmpegenc->context->pix_fmt;
+  ffmpegenc->picture->width = GST_VIDEO_FRAME_WIDTH (&buffer_info->vframe);
+  ffmpegenc->picture->height = GST_VIDEO_FRAME_HEIGHT (&buffer_info->vframe);
+
   ffmpegenc->picture->pts =
       gst_ffmpeg_time_gst_to_ff (frame->pts /
       ffmpegenc->context->ticks_per_frame, ffmpegenc->context->time_base);
@@ -664,8 +673,10 @@ gst_ffmpegvidenc_handle_frame (GstVideoEncoder * encoder,
     goto encode_fail;
 
   /* Encoder needs more data */
-  if (!have_data)
+  if (!have_data) {
+    gst_video_codec_frame_unref (frame);
     return GST_FLOW_OK;
+  }
 
   /* save stats info if there is some as well as a stats file */
   if (ffmpegenc->file && ffmpegenc->context->stats_out)
@@ -684,12 +695,10 @@ gst_ffmpegvidenc_handle_frame (GstVideoEncoder * encoder,
       pkt->size, 0, pkt->size, pkt, gst_ffmpegvidenc_free_avpacket);
   frame->output_buffer = outbuf;
 
-  /* buggy codec may not set coded_frame */
-  if (ffmpegenc->context->coded_frame) {
-    if (ffmpegenc->context->coded_frame->key_frame)
-      GST_VIDEO_CODEC_FRAME_SET_SYNC_POINT (frame);
-  } else
-    GST_WARNING_OBJECT (ffmpegenc, "codec did not provide keyframe info");
+  if (pkt->flags & AV_PKT_FLAG_KEY)
+    GST_VIDEO_CODEC_FRAME_SET_SYNC_POINT (frame);
+  else
+    GST_VIDEO_CODEC_FRAME_UNSET_SYNC_POINT (frame);
 
   return gst_video_encoder_finish_frame (encoder, frame);
 
@@ -755,8 +764,10 @@ gst_ffmpegvidenc_flush_buffers (GstFFMpegVidEnc * ffmpegenc, gboolean send)
           pkt->size, 0, pkt->size, pkt, gst_ffmpegvidenc_free_avpacket);
       frame->output_buffer = outbuf;
 
-      if (ffmpegenc->context->coded_frame->key_frame)
+      if (pkt->flags & AV_PKT_FLAG_KEY)
         GST_VIDEO_CODEC_FRAME_SET_SYNC_POINT (frame);
+      else
+        GST_VIDEO_CODEC_FRAME_UNSET_SYNC_POINT (frame);
 
       flow_ret =
           gst_video_encoder_finish_frame (GST_VIDEO_ENCODER (ffmpegenc), frame);
@@ -947,7 +958,18 @@ gst_ffmpegvidenc_register (GstPlugin * plugin)
     if (in_plugin->id == AV_CODEC_ID_RAWVIDEO ||
         in_plugin->id == AV_CODEC_ID_V210 ||
         in_plugin->id == AV_CODEC_ID_V210X ||
+        in_plugin->id == AV_CODEC_ID_V308 ||
+        in_plugin->id == AV_CODEC_ID_V408 ||
+        in_plugin->id == AV_CODEC_ID_V410 ||
         in_plugin->id == AV_CODEC_ID_R210
+        || in_plugin->id == AV_CODEC_ID_AYUV
+        || in_plugin->id == AV_CODEC_ID_Y41P
+        || in_plugin->id == AV_CODEC_ID_012V
+        || in_plugin->id == AV_CODEC_ID_YUV4
+#if AV_VERSION_INT (LIBAVCODEC_VERSION_MAJOR, LIBAVCODEC_VERSION_MINOR, LIBAVCODEC_VERSION_MICRO) >= \
+        AV_VERSION_INT (57,4,0)
+        || in_plugin->id == AV_CODEC_ID_WRAPPED_AVFRAME
+#endif
         || in_plugin->id == AV_CODEC_ID_ZLIB) {
       goto next;
     }
@@ -962,6 +984,20 @@ gst_ffmpegvidenc_register (GstPlugin * plugin)
       goto next;
     }
 
+    if (strstr (in_plugin->name, "vaapi")) {
+      GST_DEBUG
+          ("Ignoring VAAPI encoder %s. We can't handle this outside of ffmpeg",
+          in_plugin->name);
+      goto next;
+    }
+
+    if (g_str_has_suffix (in_plugin->name, "_nvenc")) {
+      GST_DEBUG
+          ("Ignoring nvenc encoder %s. We can't handle this outside of ffmpeg",
+          in_plugin->name);
+      goto next;
+    }
+
     /* only video encoders */
     if (!av_codec_is_encoder (in_plugin)
         || in_plugin->type != AVMEDIA_TYPE_VIDEO)
